from omg import *
import sys
import Image, ImageDraw

border = 4

def drawmap(wad, name, filename, maxpixels):
    edit = MapEditor(wad.maps[name])

    # determine scale = map area unit / pixel
    xmin = min([v.x for v in edit.vertexes])
    xmax = max([v.x for v in edit.vertexes])
    ymin = min([-v.y for v in edit.vertexes])
    ymax = max([-v.y for v in edit.vertexes])
    xsize = xmax - xmin
    ysize = ymax - ymin
    scale = (maxpixels-border*2) / float(max(xsize, ysize))

    # convert all numbers to image space
    xmax = int(xmax*scale); xmin = int(xmin*scale)
    ymax = int(ymax*scale); ymin = int(ymin*scale)
    xsize = int(xsize*scale); ysize = int(ysize*scale)
    for v in edit.vertexes:
        v.x = v.x * scale; v.y = -v.y * scale

    im = Image.new('RGB', (xsize+border*2, ysize+border*2), (255,255,255))
    draw = ImageDraw.Draw(im)

    # draw 1s lines after 2s lines so 1s lines are never obscured
    edit.linedefs.sort(lambda a, b: cmp(not a.two_sided, not b.two_sided))

    for line in edit.linedefs:
         p1x = edit.vertexes[line.vx_a].x - xmin + border
         p1y = edit.vertexes[line.vx_a].y - ymin + border
         p2x = edit.vertexes[line.vx_b].x - xmin + border
         p2y = edit.vertexes[line.vx_b].y - ymin + border
         color = 0, 0, 0
         if line.two_sided: color = (144, 144, 144)
         if line.action: color = (220, 130, 50)

         # draw several lines to simulate thickness
         draw.line((p1x, p1y, p2x, p2y), fill=color)
         draw.line((p1x+1, p1y, p2x+1, p2y), fill=color)
         draw.line((p1x-1, p1y, p2x-1, p2y), fill=color)
         draw.line((p1x, p1y+1, p2x, p2y+1), fill=color)
         draw.line((p1x, p1y-1, p2x, p2y-1), fill=color)

    del draw
    im.save(filename)

if (len(sys.argv) < 3):
    print "\n    Omgifol script: draw maps to image files\n"
    print "    Usage:"
    print "    drawmaps.py source.wad pattern [size]\n"
    print "    Draw all maps whose names match the given pattern (eg E?M4 or MAP*)"
else:
    print "Loading %s..." % sys.argv[1]
    inwad = WAD()
    inwad.from_file(sys.argv[1])
    try:
        maxpixels = int(sys.argv[3])
    except:
        maxpixels = 1000
    for name in inwad.maps.find(sys.argv[2]):
        print "Drawing %s" % name
        drawmap(inwad, name, name + ".png", maxpixels)
